@@ -220,25 +220,41 @@ class PD_input:
         channels = [col for col in input_file.columns if 'Abundance:' in col]
         MPA=list([col for col in input_file.columns if 'Master Protein Accession' in col])
         MPA=MPA[0]
+        PSM_grouped=input_file.groupby(by=[MPA])
+        result={}
         if method == 'sum':
-            input_file = input_file.groupby([MPA]).sum()
-            input_file.index = input_file[MPA]
-            result = input_file[channels]
-        elif method == 'median':
-            input_file = input_file.groupby([MPA]).median()
-            input_file.index = input_file[MPA]
-            result = input_file[channels]
+            for group in PSM_grouped.groups:
+                temp=PSM_grouped.get_group(group)
+                sums=temp[channels].sum()
+                result[group]=sums
         elif method == 'mean':
-            input_file = input_file.groupby([MPA]).mean()
-            input_file.index = input_file[MPA]
-            result = input_file[channels]
+            for group in PSM_grouped.groups:
+                temp=PSM_grouped.get_group(group)
+                means = temp[channels].mean()
+                result[group]=means
+
+        elif method == 'median':
+            for group in PSM_grouped.groups:
+                temp=PSM_grouped.get_group(group)
+                medians = temp[channels].median()
+                result[group] = medians
+                medians = temp[channels].median()
+                result[group] = medians
         else:
-            print('Problem detected: no correct method set. Used default')
-            input_file = input_file.groupby([MPA]).sum()
-            input_file.index = input_file[MPA]
-            result = input_file[channels]
+            for group in PSM_grouped.groups:
+                temp=PSM_grouped.get_group(group)
+                medians = temp[channels].median()
+                result[group] = medians
+                sums=temp[channels].sum()
+                result[group]=sums
         
-        return result
+
+            
+        
+        protein_df=pd.DataFrame.from_dict(result, orient='index',columns=channels)
+        
+        
+        return protein_df
 
 
     def sum_peptides_for_proteins(self,input_file):
@@ -461,26 +477,42 @@ class plain_text_input:
     def protein_rollup(self,input_file,method='sum'):
         channels = self.abundances
         MPA = self.mpa
-
+        PSM_grouped=input_file.groupby(by=[MPA])
+        result={}
         if method == 'sum':
-            input_file = input_file.groupby([MPA]).sum()
-            input_file.index = input_file[MPA]
-            result = input_file[channels]
-        elif method == 'median':
-            input_file = input_file.groupby([MPA]).median()
-            input_file.index = input_file[MPA]
-            result = input_file[channels]
+            for group in PSM_grouped.groups:
+                temp=PSM_grouped.get_group(group)
+                sums=temp[channels].sum()
+                result[group]=sums
         elif method == 'mean':
-            input_file = input_file.groupby([MPA]).mean()
-            input_file.index = input_file[MPA]
-            result = input_file[channels]
+            for group in PSM_grouped.groups:
+                temp=PSM_grouped.get_group(group)
+                means = temp[channels].mean()
+                result[group]=means
+
+        elif method == 'median':
+            for group in PSM_grouped.groups:
+                temp=PSM_grouped.get_group(group)
+                medians = temp[channels].median()
+                result[group] = medians
+                medians = temp[channels].median()
+                result[group] = medians
         else:
-            print('Problem detected: no correct method set. Used default')
-            input_file = input_file.groupby([MPA]).sum()
-            input_file.index = input_file[MPA]
-            result = input_file[channels]
+            for group in PSM_grouped.groups:
+                temp=PSM_grouped.get_group(group)
+                medians = temp[channels].median()
+                result[group] = medians
+                sums=temp[channels].sum()
+                result[group]=sums
         
-        return result
+
+            
+        
+        protein_df=pd.DataFrame.from_dict(result, orient='index',columns=channels)
+        
+        
+        return protein_df
+
 
     def sum_peptides_for_proteins(self,input):
         '''This function takes a peptide/PSM level DataFrame stored in self.input_file and performs Protein quantification rollup based
